#include <project.h>
#include <stdio.h>

/* The I2C Slave address by default in a PSoC device is 8 */
#define I2C_SLAVE_ADDRESS    (0x44)
/* Set the write buffer length to be 16 bits or 2 bytes */
#define WR_BUFFER_SIZE       (2u)


#define BUFFER_LEN  64u


char8 *parity[] = { "None", "Odd", "Even", "Mark", "Space" };
char8 *stop[] = { "1", "1.5", "2" };


void ready_USB_UART_tx(void);
uint8_t I2C_Read_Byte(uint32_t slaveAddress,uint8_t wordAddress);
void I2C_Write_Byte(uint32_t slaveAddress,uint8_t wordAddress,uint8_t WrData);

/*******************************************************************************
* Function Name: main
********************************************************************************
*
* Summary:
*  main function performs following functions:
*   1. Starts all used components.
*   2. Controls sequencing of ADC inputs.
*   3. Reads ADC converted data, sends this data via a I2C Master.
*   4. Displays ADC converted data on Character LCD.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
int main()
{
    uint8 I2C_read_buf[10];
    uint8 temp;
    uint8 sample_segment[WR_BUFFER_SIZE];
    uint16 sample_full;
    
    char tx_buf[100];
    
    uint8_t read_byte,i;

    I2CM_Start();
    USBUART_1_Start(0u, USBUART_1_3V_OPERATION);

    /* Enable global interrupts - required for I2C */
    CyGlobalIntEnable;

    ready_USB_UART_tx();
    
    // The IC measures ALS continuously
    I2C_Write_Byte(I2C_SLAVE_ADDRESS,0,0xA0);
    
    while(1)
    {
        for (i=0;i<9;i++)
        {
            I2C_read_buf[i] = I2C_Read_Byte(I2C_SLAVE_ADDRESS,i);
            CyDelay(50);
        }
        sprintf(tx_buf,"test usbuart~~[%02X][%02X][%02X][%02X][%02X][%02X][%02X][%02X]\n\r",I2C_read_buf[0],I2C_read_buf[1],I2C_read_buf[2],I2C_read_buf[3],I2C_read_buf[4],I2C_read_buf[5],I2C_read_buf[6],I2C_read_buf[7]);
        while(USBUART_1_CDCIsReady() == 0u);    // Wait till component is ready to send more data to the PC 
        USBUART_1_PutString(tx_buf);
        CyDelay(500);
    }
} /* End main */


void ready_USB_UART_tx(void)
{
    uint16 count;
    uint8 state,cnt_USB_UART_con=0;;
    uint8 buffer[BUFFER_LEN];
    char tx_buffer[BUFFER_LEN];
   /* Main Loop: */
    for(;;)
    {
        if(USBUART_1_IsConfigurationChanged() != 0u) /* Host could send double SET_INTERFACE request */
        {
            if(USBUART_1_GetConfiguration() != 0u)   /* Init IN endpoints when device configured */
            {
               /* Enumeration is done, enable OUT endpoint for receive data from Host */
                USBUART_1_CDC_Init();
            }
        }         
        if(USBUART_1_GetConfiguration() != 0u)    /* Service USB CDC when device configured */
        {
            if(USBUART_1_DataIsReady() != 0u)               /* Check for input data from PC */
            {   
                count = USBUART_1_GetAll(buffer);           /* Read received data and re-enable OUT endpoint */
                if(count != 0u)
                {
                    while(USBUART_1_CDCIsReady() == 0u);    /* Wait till component is ready to send more data to the PC */ 
                    USBUART_1_PutData(buffer, count);       /* Send data back to PC */
                    /* If the last sent packet is exactly maximum packet size, 
                    *  it shall be followed by a zero-length packet to assure the
                    *  end of segment is properly identified by the terminal.
                    */
                    if(count == BUFFER_LEN)
                    {
                        while(USBUART_1_CDCIsReady() == 0u); /* Wait till component is ready to send more data to the PC */ 
                        USBUART_1_PutData(NULL, 0u);         /* Send zero-length packet to PC */
                    }
                }
            }  
            
            state = USBUART_1_IsLineChanged();              /* Check for Line settings changed */
            if(state != 0u)
            {  
                if(state & USBUART_1_LINE_CODING_CHANGED)   /* Show new settings */
                {
                    sprintf(tx_buffer,"BR:%4ld,DB:%d\n\r",USBUART_1_GetDTERate(),(uint16)USBUART_1_GetDataBits());
                    sprintf(tx_buffer,"SB:%s,Parity:%s\n\r", stop[(uint16)USBUART_1_GetCharFormat()], \
                                                         parity[(uint16)USBUART_1_GetParityType()]);
                }

                if(state & USBUART_1_LINE_CONTROL_CHANGED)  /* Show new settings */
                {   
                    state = USBUART_1_GetLineControl();
                    sprintf(tx_buffer,"DTR:%s,RTS:%s\n\r",  (state & USBUART_1_LINE_CONTROL_DTR) ? "ON" : "OFF", \
                                                        (state & USBUART_1_LINE_CONTROL_RTS) ? "ON" : "OFF");
                    cnt_USB_UART_con++;
                    
                    if (cnt_USB_UART_con > 1)
                        break;
                }
            }
        }
    }   
}

uint8_t I2C_Read_Byte(uint32_t slaveAddress,uint8_t wordAddress)
{
    uint8_t rtn_val;
    
    I2CM_MasterWriteBuf(slaveAddress, &wordAddress, 1, I2CM_MODE_COMPLETE_XFER );
    for(;;)
	{
		if(0u != (I2CM_MasterStatus() & I2CM_MSTAT_WR_CMPLT))
		{
		 	break;
		}
	}
    
    I2CM_MasterReadBuf(slaveAddress, &rtn_val, 1, I2CM_MODE_COMPLETE_XFER);
	for(;;)
	{
        if(0u != (I2CM_MasterStatus() & I2CM_MSTAT_RD_CMPLT))
    	{
    	 	break;
    	}
	}    
    
    return rtn_val;
}

void I2C_Write_Byte(uint32_t slaveAddress,uint8_t wordAddress,uint8_t WrData)
{
    uint8_t i2c_Data[2];
    i2c_Data[0] = wordAddress;
    i2c_Data[1] = WrData;
    
    I2CM_MasterWriteBuf(slaveAddress, (uint8 *) i2c_Data, 2, I2CM_MODE_COMPLETE_XFER );
    for(;;)
	{
		if(0u != (I2CM_MasterStatus() & I2CM_MSTAT_WR_CMPLT))
		{
                
		 	break;
		}
	}
    CyDelayUs(5000);    // Write Cycle Time (5ms)
}


/* [] END OF FILE */
